#!/usr/bin/python

import argparse
import os
import sys
import yaml
import glob


# create csv file containing aggregate information
# format : date suite total broken outdated

# input : distcheck / oudated yaml output and timing
# information
# ex : $file.distcheck / $file.distcheck.time /
#      $file.outdated / $file.outdated.time

def crunch(dist,time,distcheck,outdated=None) :

    data = yaml.load(distcheck)
    broken_debcheck = data['broken-packages']
    total_packages = data['total-packages']

    if outdated :
        data = yaml.load(outdated)
        broken_outdated = data['broken-packages']

    if outdated :
        print "%s %s %s %s %s" %(time,dist,total_packages,broken_debcheck,broken_outdated)
    else :
        print "%s %s %s %s" %(time,dist,total_packages,broken_debcheck)

def main():
    parser = argparse.ArgumentParser(description='distcheck/outdated data cruncing')
    parser.add_argument('-v', '--verbose')
    parser.add_argument('-d', '--debug', action='store_true', default=False)
    parser.add_argument('--outdated', action='store_true', default=False)
    parser.add_argument('archive', type=str, nargs=1, help="archive")
    parser.add_argument('dists', type=str, nargs=1, help="dists")
    args = parser.parse_args()

    dists = args.dists[0]
    archive = args.archive[0]

    for fname in glob.iglob(archive+"/*.distcheck") :
        time = os.path.splitext(os.path.basename(fname))[0]
        if os.path.getsize(fname) > 0 :
            distcheck = open(fname)
            crunch(dists,time,distcheck)
            distcheck.close()

if __name__ == '__main__':
    main()




#!/usr/bin/python

import unittest
from subprocess import Popen, PIPE
import difflib
import uuid
import os,sys,time,glob
import argparse
from itertools import groupby, ifilter
import yaml
import copy

try :
    from yaml import CBaseLoader as yamlLoader
except ImportError:
    from yaml import BaseLoader as yamlLoader
    warning('YAML C-library not available, falling back to python')

import filecmp
import cStringIO
from sets import Set

def which(program):
    path = "/usr/share/cudf/solvers/"
    exe_file = os.path.join(path, program)
    return os.path.isfile(exe_file)

def removeTmpFiles() :
    for f in glob.glob("/tmp/apt-cudf-universe*.cudf") :
        os.remove(f)

class Ignore(Exception):
    pass

def convert(d) :
    def aux(e) :
        if isinstance(e, dict) :
            return frozenset(sorted([(k,aux(v)) for (k,v) in e.items()]))
        elif isinstance(e, list) :
            return frozenset(sorted([aux(v) for v in e]))
        else :
            return e
    return sorted([aux(e) for e in d])

def parse822(f,filter) :
    records = []
    for empty, record in groupby(ifilter(lambda s: not s.startswith('#'),open(f)), key=str.isspace):
        if not empty:
            l = map(lambda s : tuple(s.split(': ')), record)
            l = map(lambda (k,v) : (k,v.rstrip()), l)
            if (len(l) > 0):
                try :
                    pairs = ((k, filter((k,v.strip()))) for k,v in l)
                    records.append((pairs))
                except Ignore :
                    continue

    l = sorted([sorted(e) for e in records])
    return [frozenset(e) for e in l]

def parseedsp(f):
    fields = ['Package','Architecture']

    def filter(k,s,fields) :
        if k in fields :
            return s

    return parse822(f,filter)

def parsedistcheck(f) :
    cnf_fields = ['conflict','depends','provides','recommends']

    def cnf(k,s) :
        if k == "preamble" : raise Ignore
        if k in cnf_fields :
            l = s.split(',')
            ll = map(lambda s : s.split('|'), l)
            return ll
        else :
            return s

    return parse822(f,filter)

def parseyaml(f) :
    print "yaml %s" % f
    l = []
    if os.path.getsize(f) > 0 :
        data = yaml.load(open(f), Loader=yamlLoader)
        report = data.get('report', [])
        l = convert(report)
    return l

def parsetext(f) :
    l = sorted(open(f).readlines())
    return [frozenset(sorted(e)) for e in l]

def diff_aux(expectedfile,resultfile,parser):
    if filecmp.cmp(expectedfile,resultfile) :
        return True
    else :
        expected = parser(expectedfile)
        result = parser(resultfile)
        matcher = difflib.SequenceMatcher(None, expected, result)
        if matcher.ratio() == 1.0 :
            print "Warning ! Expected result and actual result are not identical."
            print "The order is not the same."
            return True
        else :
            if False :
                diff = difflib.unified_diff(open(expectedfile).readlines(),open(resultfile).readlines())
                sys.stdout.writelines(list(diff))
            return False

def diff_yaml(expectedfile,resultfile):
    return diff_aux(expectedfile,resultfile,parseyaml)

def diff_822(expectedfile,resultfile):
    return diff_aux(expectedfile,resultfile,parsedistcheck)

def diff_text(expectedfile,resultfile):
    return diff_aux(expectedfile,resultfile,parsetext)

def diff_edsp(expectedfile,resultfile):
    return diff_aux(expectedfile,resultfile,parseedsp)
    
def test_application(self,expected_file,cmd,diff,exitcode):
    uid = uuid.uuid1()
    mytmp = False
    if not os.path.exists("tmp"):
        mytmp = True
        os.makedirs("tmp")

    output_file = "tmp/%s.cudf" % uid
    output = open(output_file,'w')
    p = Popen(cmd, stdout=output)
    p.communicate()
    rc = p.returncode if exitcode is not None else None
    if rc == exitcode :
        ec = True
    else :
        print "ExitCode = %d" % rc
        ec = False
    d = diff(expected_file,output_file)
    output.close()
    os.remove(output_file)
    if mytmp :
        os.rmdir("tmp")
    self.assertTrue(d)
    self.assertTrue(ec)

class DoseTests(unittest.TestCase):
    def __init__(self, test):
        super(DoseTests, self).__init__()
        self.name = test['Name'] 
        self.group = test['Group']
        self.comment = test['Comment'] if 'Comment' in test else None
        self.expected = test['Expected'] 
        self.cmd = test['Cmd'].split(' ') + test['Input'].split(' ')
        self.exitcode = int(test['ExitCode']) if 'ExitCode' in test else None
        self.enctype = test['Type'] if 'Type' in test else '822'
        self.solver = test['Solver'] if 'Solver' in test else None
        if self.enctype == '822' :
            self.difftype = diff_822
        elif self.enctype  == 'yaml' :
            self.difftype = diff_yaml
        elif self.enctype  == 'text' :
            self.difftype = diff_text
        elif self.enctype  == 'edsp' :
            self.difftype = diff_text
        else :
            self.difftype = diff_text
    def shortDescription(self):
        if self.comment :
            return "Description : " + self.comment + "\n" + "\nExpected file : %s" % self.expected + "\n"
        else :
            s =     "Test : %s" % self.name
            s = s + "\nGroup : %s" % self.group
            s = s + "\n" + "Cmd : " + " ".join(self.cmd)
            if self.solver : 
                s = s + "\n" + "Solver : " + self.solver
            s = s + "\nExpected file : %s" % self.expected
            s = s + "\nExpected exitcode : %d" % self.exitcode if self.exitcode is not None else s
            return s + "\n"
    def runTest(self):
        test_application(self,self.expected,self.cmd,self.difftype,self.exitcode)
    def tearDown(self):
        removeTmpFiles()

def suite(f,runtest,rungroup,slow=False):
    suite = unittest.TestSuite()
    groups = Set()
    tests = Set()
    groupFound=False
    testsFound=False
    def addtest(s) :
        # we run the slow tests only if 
        if (slow and 'Speed' in s and s['Speed'] == 'slow') :
            return
        else :
            #default we run the test
            if 'edsp' in s['Type'] :
                solvers = ['aspcud','packup','mccs-lpsolve']
                if 'Solver' in s :
                    solvers = [x.strip() for x in s['Solver'].split(',')]
                for solver in solvers :
                    ss = copy.deepcopy(s)
                    f = "%s-%s" % (ss['Expected'],solver)
                    ss['Expected'] = f if os.path.isfile(f) else ss['Expected']
                    ss['Solver'] = solver
                    ss['Cmd'] = ss['Cmd'] + " --solver " + solver
                    suite.addTest(DoseTests(ss))
            else :
                suite.addTest(DoseTests(s))
    for stanza in parse822(f,lambda s: s[1]):
        s = dict(stanza)
        if s['Name'] not in runtest and 'Ignore' in s and s['Ignore'] == 'yes' :
            continue
        groups.add(s['Group'])
        if (len(runtest) == 0 and len(rungroup) == 0) :
            addtest(s)
        elif s['Name'] in runtest :
            testFound=True
            addtest(s)
        elif len(rungroup) > 0 and s['Group'] in rungroup :
            groupFound=True
            addtest(s)
    if len(runtest) != 0 and testFound == False :
        print "Test(s) [%s] Not found" % (','.join(str(p) for p in runtest)) 
        print "Tests available [%s]" % (','.join(str(p) for p in tests))
    if len(rungroup) != 0 and groupFound == False :
        print "Group(s) [%s] Not found" % (','.join(str(p) for p in rungroup))
        print "Groups available [%s]" % (','.join(str(p) for p in groups))

    return suite

def fixtest(expected_file,cmd):
    output = open(expected_file,'w')
    p = Popen(cmd, stdout=output)
    p.communicate()
    output.close()

def main():
    parser = argparse.ArgumentParser(description='Unit test for Dose applications')
    parser.add_argument('-d', '--diff', action='store_true', default=False, help="Print diffs")
    parser.add_argument('-s', '--slow', action='store_true', default=False, help="Run slow tests") 
    parser.add_argument('--runtest', nargs=1, default=[]) 
    parser.add_argument('--rungroup', nargs=1, default=[]) 
    parser.add_argument('--fixtest', nargs=1, default=[]) 
    parser.add_argument('inputfile', type=str, nargs=1, help="test file")
    args = parser.parse_args()

    if len(args.fixtest) > 0 :
        f = args.inputfile[0]
        for stanza in parse822(f,lambda s: s[1]):
            s = dict(stanza)
            if args.fixtest[0] == s['Name'] :
                cmd = s['Cmd'].split(' ') + s['Input'].split(' ')
                expected = s['Expected']
                print "Overwriting expected file: %s" % expected
                fixtest(expected,cmd)
    else :
        unittest.TextTestRunner(verbosity=2).run(suite(args.inputfile[0],args.runtest,args.rungroup,args.slow))

if __name__ == '__main__':
    main()




#!/usr/bin/python
import argparse
import csv
import datetime as dt
import matplotlib.pyplot as plt
from matplotlib import dates
import os.path
from itertools import groupby
from operator import itemgetter, attrgetter, methodcaller
import collections

# takes as a input a cvs file and a list of distributions and
# creates a plot of the evolution of broken and outdated packages
# vs time.

# cvs format "date suite packages broken (outdated)"
# ex : 20111125 unstable 35056 337 (73)

# parse a time line file of the form :
# Title: ...
# Date: Mar 25 2013
# <other infos>: ignored
def parse_timeline(fname,timeline) :
    records = []
    for empty, record in groupby(open(fname), key=str.isspace):
      if not empty:
        pairs = map(lambda s : s.split(': '), record)
        pairs = dict(map(lambda (k,v) : (k,v.rstrip()), pairs))
        pairs['Date'] = dt.datetime.strptime(pairs['Date'],"%b %d %Y")
        pairs['Timeline'] = timeline
        records.append(pairs)
    return records

def aggregate_timeline(timelines) :
    t = sorted(timelines,key=lambda t : t['Date'])
    l = []
    i=0
    while i < len(t) - 1 :
        if t[i]['Timeline'] == 'release' :
            if t[i+1]['Timeline'] == 'release' :
                #l.append(t[i])
                i=i+1
            else :
                i=i+1
        elif t[i]['Timeline'] == 'freeze' :
            if t[i+1]['Timeline'] == 'release' :
                l.append((t[i],t[i+1]))
                i=i+2
            else :
                i=i+1
        else :
            i=i+1
    return l

def initdict(dists,distname) :
    dists[distname] = {'date' : [], 'total' : [], 'broken' : [], 'outdated' : [] }
    return dists

def parse(dataset,outdated=False):
    dists = collections.OrderedDict()
    csv_reader = csv.reader(dataset,delimiter=' ')
    for line in sorted(csv_reader,key=lambda k: k[0]):
        distname = line[1]
        if distname not in dists :
            dists = initdict(dists,distname)

        dists[distname]['date'].append(dt.datetime.strptime(line[0],'%Y%m%d'))
        dists[distname]['total'].append(int(line[2]))
        dists[distname]['broken'].append(int(line[3]))
        if outdated : 
            dists[distname]['outdated'].append(int(line[4]))

    return dists

def plot(dists,output,title,intervals,outdated=False) :

    distlist = dists.keys()
    fig = plt.figure()
    fig.suptitle(title)
    plotsidx = [311,312,313] if outdated else [211,212]
    ax1 = fig.add_subplot(plotsidx[0],title='Total Packages vs Time')
    ax2 = fig.add_subplot(plotsidx[1],title='Non-Installable Packages vs Time')
    if outdated : 
        ax3 = fig.add_subplot(plotsidx[2],title='Outdated Packages vs Time')
    for k in distlist :
        ax1.plot(dists[k]['date'],dists[k]['total'],',-',label=k.capitalize())
        ax2.plot(dists[k]['date'],dists[k]['broken'],',-',label=k.capitalize())
        if intervals :
            mdate = min(dists[k]['date'])
            for (f,r) in intervals :
                if f['Date'] > mdate :
                    ax2.axvspan(f['Date'],r['Date'], alpha=0.5, color='red')
        if outdated :
            ax3.plot(dists[k]['date'],dists[k]['outdated'],',-',label=k.capitalize())


    if len(distlist) > 1 :
        ax1.legend(loc='upper left')
        ax2.legend(loc='upper left')
        #ax2.legend()
        if outdated :
            ax3.legend(loc='upper left')

    fig.autofmt_xdate()

    plt.savefig(output)

def multiplot(dists,output,title,timelines,intervals) :

    fig = plt.figure()
    fig.suptitle(title)
    subplotnum = len(dists)
    #fig.set_figheight((subplotnum+1) * 3)
    fig.set_figheight(24)

    ax = {}
    plotsidx = []
    for i in range(1, subplotnum+2) :
        plotsidx.append(int(("%d1%d") % (subplotnum+1,i)))

    print plotsidx

    # First graph puts all dists together in one
    ax[0] = fig.add_subplot(plotsidx[0],title='Total Packages vs Time')
    for k in dists :
        ax[0].plot(dists[k]['date'],dists[k]['total'],',-',label=k.capitalize())
    ax[0].legend(loc='upper left')

    i = 1
    for n in dists.keys() :
        ax[i] = fig.add_subplot(plotsidx[i])
        if i == 1 :
            ax[i].set_title('Non-Installable Packages vs Time')
        ax[i].plot(dists[n]['date'],dists[n]['broken'],',-',label=n.capitalize())
        ax[i].legend(loc='upper left')
        if intervals :
            mdate = min(dists[k]['date'])
            for (f,r) in intervals :
                if f['Date'] > mdate :
                    ax[i].axvspan(f['Date'],r['Date'], alpha=0.5, color='red')
        i += 1

    fig.autofmt_xdate()
    plt.savefig(output)

# plot two distribution with different scales
def multiscale(dists,dist1,dist2,output,title,outdated=False) :

    fig = plt.figure()
    fig.suptitle(title)
    fig.autofmt_xdate()

    plotsidx = [311,312,313] if outdated else [211,212]
    ax1 = fig.add_subplot(plotsidx[0],title='Total Packages vs Time')
    ax1.plot(dists[dist1]['date'],dists[dist1]['total'],',-',label=dist1.capitalize())
    ax1.plot(dists[dist2]['date'],dists[dist2]['total'],',-',label=dist2.capitalize())
    ax1.legend(loc='upper left')
    ax1.xaxis.set_visible(False)

    ax2 = fig.add_subplot(plotsidx[1],title='Non-Installable Packages vs Time')
    ax2.plot(dists[dist1]['date'],dists[dist1]['broken'],'o-',label=dist1.capitalize())
    ax2.xaxis.set_visible(False)

    ax22 = ax2.twinx() 
    ax22.plot(dists[dist2]['date'],dists[dist2]['broken'],'s-',label=dist2.capitalize())
    ax22.set_ylim(0, 70)

    if outdated : 
        ax3 = fig.add_subplot(plotsidx[2],title='Outdated Packages vs Time')
        ax3.plot(dists[dist1]['date'],dists[dist1]['outdated'],'o-',label=dist1.capitalize())

        ax33 = ax3.twinx() 
        ax33.plot(dists[dist2]['date'],dists[dist2]['outdated'],'gs-',label=dist2.capitalize())
        ax33.set_ylim(0, 10)
        # ax33.set_yticks([-1,0,1])

        plt.setp(ax3.xaxis.get_majorticklabels(), rotation=30)

    plt.savefig(output)

def main():
    parser = argparse.ArgumentParser(description='plot outdated/broken')
    parser.add_argument('-v', '--verbose')
    parser.add_argument('-d', '--debug', action='store_true', default=False)
    parser.add_argument('-o', '--output', action='store')
    parser.add_argument('-t', '--title', action='store')
    parser.add_argument('-s', '--split', action='store_true', default=False)
    parser.add_argument('--ms', action='store_true', default=False, help="multi plot")
    parser.add_argument('--mp', action='store_true', default=False, help="multi scale")
    parser.add_argument('--releases', type=str, nargs=1, help="release timeline")
    parser.add_argument('--freezes', type=str, nargs=1, help="freeze timeline")
    parser.add_argument('--outdated', action='store_true', default=False)
    parser.add_argument('dataset', type=str, nargs=1, help="dataset")
    args = parser.parse_args()
 
    fname = args.dataset[0]

    dataset = open(fname)
    dists = parse(dataset)
    dataset.close()

    title = args.title[0] if args.title else ""

    timelines = {}
    intervals = []
    if args.releases :
        timelines['releases'] = parse_timeline(args.releases[0],'release')
    if args.freezes :
        timelines['freezes'] = parse_timeline(args.freezes[0],'freeze')
    if args.releases and args.freezes :
        intervals = aggregate_timeline(timelines['releases'] + timelines['freezes'])

    if args.ms :
        output = "aggregate-%s-ms.png" % os.path.splitext(os.path.basename(fname))[0]
        multiscale(dists, output, title, args.outdated)
    elif args.mp :
        output = "aggregate-%s-mp.png" % os.path.splitext(os.path.basename(fname))[0]
        multiplot(dists, output, title, timelines, intervals)
    else :
        if args.split :
            # different graphs, one for each suite
            for d in dists.keys() :
                output = "%s-%s.png" % (d,os.path.splitext(os.path.basename(fname))[0])
                plot(dists, [d] , output, title, intervals, args.outdated)
        else :
            # all in one graph
            output = "aggregate-%s.png" % os.path.splitext(os.path.basename(fname))[0]
            plot(dists, output, title, intervals, args.outdated)

if __name__ == '__main__':
    main()

